/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.plugin;

import com.sos.scheduler.engine.kernel.log.PrefixLog;
import static com.sos.scheduler.engine.kernel.util.XmlUtils.xmlQuoted;

/** Die Engine spricht das Plugin über diesen Adapter an. */
class PluginAdapter {
    private final Plugin plugin;
    private final String name;
    private final PrefixLog log;

    PluginAdapter(Plugin plugin, String name, PrefixLog log) {
        this.plugin = plugin;
        this.name = name;
        this.log = log;
    }

    final void activate() {
        plugin.activate();
    }

    final String getXmlState() {
        try {
            return plugin.getXmlState();
        } catch (Exception x) {
            return "<ERROR text=" + xmlQuoted(x.toString()) + "/>";
        }
    }

    final void tryActivate() {
        try {
            plugin.activate();
        } catch (Exception x) {
            logThrowable(x);
        }
    }

    final void tryClose() {
        try {
            plugin.close();
        } catch (Exception x) {
            logThrowable(x);
        }
    }

    private void logThrowable(Throwable t) {
        PluginSubsystem.logError(log, toString(), t);
    }

    final String getPluginClassName() {
        return plugin.getClass().getName();
    }

    final Plugin getPlugin() {
        return plugin;
    }

    @Override public String toString() {
        return "Plugin " + name;
    }
}
