/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

public final class Time {
    public static final Time eternal = new Time(Long.MAX_VALUE, TimeUnit.DAYS);
    
    public final long value;
    public final TimeUnit unit;

    public Time(long value, TimeUnit timeUnit) {
        this.value = value;
        this.unit = timeUnit;
    }

    public long getMillis() {
        return unit.toMillis(value);
    }

//    public long getNanosWithoutMillis() {
//        return unit.excessNanos(getMillis(), value);
//    }

    public final String toXml() {
        return new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").format(new Date(getMillis()));
    }

    @Override public String toString() {
        return (unit.toMillis(value) / 1000.0) + "s";
    }


    public static Time of(long value, TimeUnit u) {
        return new Time(value, u);
    }

    public static Time of(double seconds) {
        return new Time((long)(seconds * 1000000000), TimeUnit.NANOSECONDS);
    }

    public static Time of(Date o) {
        return new Time(o.getTime(), TimeUnit.MILLISECONDS);
    }
    //public static Time of(Date d) { return ofMillis(d.getTime()); }

    public static Time ofMillis(long value) {
        return new Time(value, TimeUnit.MILLISECONDS);
    }
}
