/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package net.cwroethel.swt.popupcalendar;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

/*******************************************************************************
 * Copyright (c) 2005 Will Roethel. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html Contributors: Will Roethel -
 * initial API and implementation
 ******************************************************************************/

/**
 * Style Class containing the default style for the GUI components used in the
 * DaySelector as well as accessors to the component templates. This class is
 * meant to be extended by users longing to set their own definitions. The
 * preferred way to do that is to inherit constructors and all methods from this
 * class and just implement the overrideStyle() class. This allows the
 * definition of personal settings without having to deal with the internals of
 * the style class. For additional conveniance a DaySelectorStyleAdapter is
 * provided.
 * 
 * @author Will Roethel
 * @version $Revision: 1.2 $
 */
public class DaySelectorStyle extends DateChooserStyle {
    /**
     * The style identifiers for the different MonthSelector components
     */
    public final static int   WEEKDAYLABEL       = 1;

    public final static int   BLANKLABEL         = 2;

    public final static int   DAYLABEL           = 3;

    public final static int   TODAYLABEL         = 4;

    public final static int   SELECTEDLABEL      = 5;

    /**
     * True if the weekday labels should be shown.
     */
    public boolean            showWeekdays       = true;

    /**
     * String[] containing the names of the weekdays to be shown.
     */
    protected String[]        weekdayName        = null;

    protected static String[] weekdayNameDefault = { "S", "M", "T", "W", "T", "F", "S" };


    /**
     * Set the default values
     */
    protected final void setDefaultStyle(Control control, int style) {
        Label label = (Label) control;
        ((GridData) label.getLayoutData()).widthHint = daySize;
        ((GridData) label.getLayoutData()).heightHint = daySize;

        switch (style) {
            case WEEKDAYLABEL:
                label.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
                label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
                break;

            case BLANKLABEL:
                label.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
                // label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));
                label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_TITLE_INACTIVE_FOREGROUND));
                break;

            case DAYLABEL:
                label.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
                label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
                break;

            case SELECTEDLABEL:
                label.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW));
                label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
                break;

            case TODAYLABEL:
                label.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));
                label.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
                break;
        }
    }


    /**
     * Get the weekday names used for the weekday labels.
     * 
     * @param dayOfTheWeek int
     * @return String
     */
    public final String getWeekdayName(int dayOfTheWeek) {
        return weekdayName[dayOfTheWeek];
    }


    /**
     * Get the array containing the weekday names in the current locale.
     * 
     * @return String[]
     */
    public final String[] getWeekdayNames() {
        if (weekdayName == null) {
            try {
                setWeekdayNames(CalendarNamesUtil.getWeekdayNames(locale));
            } catch (Exception e) {
                weekdayName = weekdayNameDefault;
            }
        }
        return weekdayName;
    }


    /**
     * Override the default weekday names with a new set. The array must be of
     * length 7 otherwise setWeekdayNames will throw an exception.
     * 
     * @param weekdays String[]
     * @throws Exception
     */

    public final void setWeekdayNames(String[] weekdays) throws Exception {
        if (weekdays.length != 7) {
            throw new Exception("Expected String[] with length 7. Found length " + weekdays.length
                    + ". Keeping the old settings.");
        }
        weekdayName = weekdays;
    }
}
