/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.LaunchAndObserve;

import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.JSHelper.Options.SOSOptionHostName;
import com.sos.JSHelper.Options.SOSOptionMailAdress;
import com.sos.JSHelper.Options.SOSOptionPassword;
import com.sos.JSHelper.Options.SOSOptionPortNumber;
import com.sos.JSHelper.Options.SOSOptionString;

/**
 * \class 		SOSSmtpMailOptionsSuperClass - General SmtP Options
 *
 * \brief 
 * An Options-Super-Class with all Options. This Class will be extended by the "real" Options-class (\see SOSSmtpMailOptions.
 * The "real" Option class will hold all the things, which are normaly overwritten at a new generation
 * of the super-class.
 *
 *

 *
 * see \see C:\Users\KB\Documents\xmltest\JobSchedulerSmtpMail.xml for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by  from http://www.sos-berlin.com at 20111124194149 
 * \endverbatim
 * \section OptionsTable Tabelle der vorhandenen Optionen
 * 
 * Tabelle mit allen Optionen
 * 
 * MethodName
 * Title
 * Setting
 * Description
 * IsMandatory
 * DataType
 * InitialValue
 * TestValue
 * 
 * 
 *
 * \section TestData Eine Hilfe zum Erzeugen einer HashMap mit Testdaten
 *
 * Die folgenden Methode kann verwendet werden, um fr einen Test eine HashMap
 * mit sinnvollen Werten fr die einzelnen Optionen zu erzeugen.
 *
 * \verbatim
 private HashMap <String, String> SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM) {
	pobjHM.put ("		SOSSmtpMailOptionsSuperClass.auth_file", "test");  // This parameter specifies the path and name of a user's pr
		return pobjHM;
  }  //  private void SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM)
 * \endverbatim
 */
@JSOptionClass(name = "SOSSmtpMailOptionsSuperClass", description = "SOSSmtpMailOptionsSuperClass")
public class SOSSmtpMailOptionsSuperClass extends JSOptionsClass implements ISOSSmtpMailOptions {
	private final String	conClassName	= "SOSSmtpMailOptionsSuperClass";
	@SuppressWarnings("unused")
	private static Logger	logger			= Logger.getLogger(SOSSmtpMailOptionsSuperClass.class);

	/**
	 * \var attachment : title Filename and pat
	 * Filename and path of the attachment(s) (multiple attachments separated by ";")
	 *
	 */
	@JSOptionDefinition(name = "attachment", description = "title Filename and pat", key = "attachment", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	attachment		= new SOSOptionString(this, conClassName + ".attachment", // HashMap-Key
													"title Filename and pat", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getattachment
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getattachment() {
		return attachment;
	}

	/**
	 * \brief setattachment
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_attachment
	 */
	@Override
	public void setattachment(SOSOptionString p_attachment) {
		this.attachment = p_attachment;
	}

	/**
	 * \var attachment_charset : title charset of attac
	 * charset of attachments
	 *
	 */
	@JSOptionDefinition(name = "attachment_charset", description = "title charset of attac", key = "attachment_charset", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	attachment_charset	= new SOSOptionString(this, conClassName + ".attachment_charset", // HashMap-Key
														"title charset of attac", // Titel
														"iso-8859-1", // InitValue
														"iso-8859-1", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getattachment_charset
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getattachment_charset() {
		return attachment_charset;
	}

	/**
	 * \brief setattachment_charset
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_attachment_charset
	 */
	@Override
	public void setattachment_charset(SOSOptionString p_attachment_charset) {
		this.attachment_charset = p_attachment_charset;
	}

	/**
	 * \var attachment_content_type : title content_type of
	 * content_type of attachments (application/octet-stream, application/pdf...)
	 *
	 */
	@JSOptionDefinition(name = "attachment_content_type", description = "title content_type of", key = "attachment_content_type", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	attachment_content_type	= new SOSOptionString(this, conClassName + ".attachment_content_type", // HashMap-Key
															"title content_type of", // Titel
															"application/octet-stream", // InitValue
															"application/octet-stream", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getattachment_content_type
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getattachment_content_type() {
		return attachment_content_type;
	}

	/**
	 * \brief setattachment_content_type
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_attachment_content_type
	 */
	@Override
	public void setattachment_content_type(SOSOptionString p_attachment_content_type) {
		this.attachment_content_type = p_attachment_content_type;
	}

	/**
	 * \var attachment_encoding : title encoding of atta
	 * encoding of attachments (7bit, Quoted-Printable, Base64)
	 *
	 */
	@JSOptionDefinition(name = "attachment_encoding", description = "title encoding of atta", key = "attachment_encoding", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	attachment_encoding	= new SOSOptionString(this, conClassName + ".attachment_encoding", // HashMap-Key
														"title encoding of atta", // Titel
														"Base64", // InitValue
														"Base64", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getattachment_encoding
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getattachment_encoding() {
		return attachment_encoding;
	}

	/**
	 * \brief setattachment_encoding
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_attachment_encoding
	 */
	@Override
	public void setattachment_encoding(SOSOptionString p_attachment_encoding) {
		this.attachment_encoding = p_attachment_encoding;
	}

	/**
	 * \var bcc : title bcc recipient(s)
	 * bcc recipient(s)
	 *
	 */
	@JSOptionDefinition(name = "bcc", description = "title bcc recipient(s)", key = "bcc", type = "SOSOptionMailAdress", mandatory = false)
	public SOSOptionMailAdress	bcc	= new SOSOptionMailAdress(this, conClassName + ".bcc", // HashMap-Key
											"title bcc recipient(s)", // Titel
											"", // InitValue
											"", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getbcc
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionMailAdress getbcc() {
		return bcc;
	}

	/**
	 * \brief setbcc
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_bcc
	 */
	@Override
	public void setbcc(SOSOptionMailAdress p_bcc) {
		this.bcc = p_bcc;
	}

	/**
	 * \var body : title Mail body
	 * Mail body
	 *
	 */
	@JSOptionDefinition(name = "body", description = "title Mail body", key = "body", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	body	= new SOSOptionString(this, conClassName + ".body", // HashMap-Key
											"title Mail body", // Titel
											"", // InitValue
											"", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getbody
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getbody() {
		return body;
	}

	/**
	 * \brief setbody
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_body
	 */
	@Override
	public void setbody(SOSOptionString p_body) {
		this.body = p_body;
	}

	/**
	 * \var cc : title cc recipient(s)
	 * cc recipient(s)
	 *
	 */
	@JSOptionDefinition(name = "cc", description = "title cc recipient(s)", key = "cc", type = "SOSOptionMailAdress", mandatory = false)
	public SOSOptionMailAdress	cc	= new SOSOptionMailAdress(this, conClassName + ".cc", // HashMap-Key
											"title cc recipient(s)", // Titel
											"", // InitValue
											"", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getcc
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionMailAdress getcc() {
		return cc;
	}

	/**
	 * \brief setcc
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_cc
	 */
	@Override
	public void setcc(SOSOptionMailAdress p_cc) {
		this.cc = p_cc;
	}

	/**
	 * \var charset : title charset of the m
	 * charset of the mail
	 *
	 */
	@JSOptionDefinition(name = "charset", description = "title charset of the m", key = "charset", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	charset	= new SOSOptionString(this, conClassName + ".charset", // HashMap-Key
											"title charset of the m", // Titel
											"iso-8859-1", // InitValue
											"iso-8859-1", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getcharset
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getcharset() {
		return charset;
	}

	/**
	 * \brief setcharset
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_charset
	 */
	@Override
	public void setcharset(SOSOptionString p_charset) {
		this.charset = p_charset;
	}

	/**
	 * \var content_type : title content_type of
	 * content_type of the mail (text/plain, text/html...)
	 *
	 */
	@JSOptionDefinition(name = "content_type", description = "title content_type of", key = "content_type", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	content_type	= new SOSOptionString(this, conClassName + ".content_type", // HashMap-Key
													"title content_type of", // Titel
													"text/plain", // InitValue
													"text/plain", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getcontent_type
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getcontent_type() {
		return content_type;
	}

	/**
	 * \brief setcontent_type
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_content_type
	 */
	@Override
	public void setcontent_type(SOSOptionString p_content_type) {
		this.content_type = p_content_type;
	}

	/**
	 * \var encoding : title encoding of the
	 * encoding of the mail (7bit, Quoted-Printable, Base64)
	 *
	 */
	@JSOptionDefinition(name = "encoding", description = "title encoding of the", key = "encoding", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	encoding	= new SOSOptionString(this, conClassName + ".encoding", // HashMap-Key
												"title encoding of the", // Titel
												"7bit", // InitValue
												"7bit", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getencoding
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getencoding() {
		return encoding;
	}

	/**
	 * \brief setencoding
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_encoding
	 */
	@Override
	public void setencoding(SOSOptionString p_encoding) {
		this.encoding = p_encoding;
	}

	/**
	 * \var from : title mail sender
	 * mail sender
	 *
	 */
	@JSOptionDefinition(name = "from", description = "title mail sender", key = "from", type = "SOSOptionMailAdress", mandatory = false)
	public SOSOptionMailAdress	from	= new SOSOptionMailAdress(this, conClassName + ".from", // HashMap-Key
												"title mail sender", // Titel
												"", // InitValue
												"", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getfrom
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionMailAdress getfrom() {
		return from;
	}

	/**
	 * \brief setfrom
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_from
	 */
	@Override
	public void setfrom(SOSOptionMailAdress p_from) {
		this.from = p_from;
	}

	/**
	 * \var from_name : title name of the send
	 * name of the sender
	 *
	 */
	@JSOptionDefinition(name = "from_name", description = "title name of the send", key = "from_name", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	from_name	= new SOSOptionString(this, conClassName + ".from_name", // HashMap-Key
												"title name of the send", // Titel
												"", // InitValue
												"", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getfrom_name
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getfrom_name() {
		return from_name;
	}

	/**
	 * \brief setfrom_name
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_from_name
	 */
	@Override
	public void setfrom_name(SOSOptionString p_from_name) {
		this.from_name = p_from_name;
	}

	/**
	 * \var host : title mail server host
	 * mail server host
	 *
	 */
	@JSOptionDefinition(name = "host", description = "title mail server host", key = "host", type = "SOSOptionHostName", mandatory = false)
	public SOSOptionHostName	host	= new SOSOptionHostName(this, conClassName + ".host", // HashMap-Key
												"title mail server host", // Titel
												"", // InitValue
												"", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief gethost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionHostName gethost() {
		return host;
	}

	/**
	 * \brief sethost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_host
	 */
	@Override
	public void sethost(SOSOptionHostName p_host) {
		this.host = p_host;
	}

	/**
	 * \var port : title mail server port
	 * mail server port
	 *
	 */
	@JSOptionDefinition(name = "port", description = "title mail server port", key = "port", type = "SOSOptionPortNumber", mandatory = false)
	public SOSOptionPortNumber	port	= new SOSOptionPortNumber(this, conClassName + ".port", // HashMap-Key
												"title mail server port", // Titel
												"25", // InitValue
												"25", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getport
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPortNumber getport() {
		return port;
	}

	/**
	 * \brief setport
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_port
	 */
	@Override
	public void setport(SOSOptionPortNumber p_port) {
		this.port = p_port;
	}

	/**
	 * \var queue_directory : title Mail queue direc
	 * Mail queue directory. Mails which cannot be transferred will be put here. The Job Scheduler will later retry to send these mails.
	 *
	 */
	@JSOptionDefinition(name = "queue_directory", description = "title Mail queue direc", key = "queue_directory", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	queue_directory	= new SOSOptionString(this, conClassName + ".queue_directory", // HashMap-Key
													"title Mail queue direc", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getqueue_directory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getqueue_directory() {
		return queue_directory;
	}

	/**
	 * \brief setqueue_directory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_queue_directory
	 */
	@Override
	public void setqueue_directory(SOSOptionString p_queue_directory) {
		this.queue_directory = p_queue_directory;
	}

	/**
	 * \var reply_to : title reply address
	 * reply address
	 *
	 */
	@JSOptionDefinition(name = "reply_to", description = "title reply address", key = "reply_to", type = "SOSOptionMailAdress", mandatory = false)
	public SOSOptionMailAdress	reply_to	= new SOSOptionMailAdress(this, conClassName + ".reply_to", // HashMap-Key
													"title reply address", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getreply_to
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionMailAdress getreply_to() {
		return reply_to;
	}

	/**
	 * \brief setreply_to
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_reply_to
	 */
	@Override
	public void setreply_to(SOSOptionMailAdress p_reply_to) {
		this.reply_to = p_reply_to;
	}

	/**
	 * \var smtp_password : title smtp user passwo
	 * smtp user password
	 *
	 */
	@JSOptionDefinition(name = "smtp_password", description = "title smtp user passwo", key = "smtp_password", type = "SOSOptionPassword", mandatory = false)
	public SOSOptionPassword	smtp_password	= new SOSOptionPassword(this, conClassName + ".smtp_password", // HashMap-Key
														"title smtp user passwo", // Titel
														"", // InitValue
														"", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getsmtp_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPassword getsmtp_password() {
		return smtp_password;
	}

	/**
	 * \brief setsmtp_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_smtp_password
	 */
	@Override
	public void setsmtp_password(SOSOptionPassword p_smtp_password) {
		this.smtp_password = p_smtp_password;
	}

	/**
	 * \var smtp_user : title smtp username
	 * smtp username
	 *
	 */
	@JSOptionDefinition(name = "smtp_user", description = "title smtp username", key = "smtp_user", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	smtp_user	= new SOSOptionString(this, conClassName + ".smtp_user", // HashMap-Key
												"title smtp username", // Titel
												"", // InitValue
												"", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getsmtp_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getsmtp_user() {
		return smtp_user;
	}

	/**
	 * \brief setsmtp_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_smtp_user
	 */
	@Override
	public void setsmtp_user(SOSOptionString p_smtp_user) {
		this.smtp_user = p_smtp_user;
	}

	/**
	 * \var subject : title Mail Subject
	 * Mail Subject
	 *
	 */
	@JSOptionDefinition(name = "subject", description = "title Mail Subject", key = "subject", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	subject	= new SOSOptionString(this, conClassName + ".subject", // HashMap-Key
											"title Mail Subject", // Titel
											"", // InitValue
											"", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getsubject
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getsubject() {
		return subject;
	}

	/**
	 * \brief setsubject
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_subject
	 */
	@Override
	public void setsubject(SOSOptionString p_subject) {
		this.subject = p_subject;
	}

	/**
	 * \var to : title mail recipient(s
	 * mail recipient(s)
	 *
	 */
	@JSOptionDefinition(name = "to", description = "title mail recipient(s", key = "to", type = "SOSOptionMailAdress", mandatory = true)
	public SOSOptionMailAdress	to	= new SOSOptionMailAdress(this, conClassName + ".to", // HashMap-Key
											"title mail recipient(s", // Titel
											"", // InitValue
											"", // DefaultValue
											true // isMandatory
									);

	/**
	 * \brief getto
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionMailAdress getto() {
		return to;
	}

	/**
	 * \brief setto
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_to
	 */
	@Override
	public void setto(SOSOptionMailAdress p_to) {
		this.to = p_to;
	}

	public SOSSmtpMailOptionsSuperClass() {
		objParentClass = this.getClass();
	} // public SOSSmtpMailOptionsSuperClass

	public SOSSmtpMailOptionsSuperClass(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public SOSSmtpMailOptionsSuperClass

	//

	public SOSSmtpMailOptionsSuperClass(HashMap<String, String> JSSettings) throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public SOSSmtpMailOptionsSuperClass (HashMap JSSettings)

	/**
	 * \brief getAllOptionsAsString - liefert die Werte und Beschreibung aller
	 * Optionen als String
	 *
	 * \details
	 * 
	 * \see toString 
	 * \see toOut
	 */
	private String getAllOptionsAsString() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getAllOptionsAsString";
		String strT = conClassName + "\n";
		final StringBuffer strBuffer = new StringBuffer();
		// strT += IterateAllDataElementsByAnnotation(objParentClass, this,
		// JSOptionsClass.IterationTypes.toString, strBuffer);
		// strT += IterateAllDataElementsByAnnotation(objParentClass, this, 13,
		// strBuffer);
		strT += this.toString(); // fix
		//
		return strT;
	} // private String getAllOptionsAsString ()

	/**
	 * \brief setAllOptions
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	@Override
	public void setAllOptions(HashMap<String, String> pobjJSSettings) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	/**
	 * \brief CheckMandatory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @throws JSExceptionMandatoryOptionMissing
	 * @throws Exception
	 */
	@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
			, Exception {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 * \brief CommandLineArgs
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) throws Exception {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class SOSSmtpMailOptionsSuperClass
