/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.file;
import java.io.File;
import java.util.HashMap;
import java.util.Vector;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import sos.scheduler.job.JobSchedulerJobAdapter;
import sos.spooler.Job_chain;
import sos.spooler.Order;
import sos.spooler.Variable_set;
import sos.util.SOSFileOperations;
import sos.util.SOSSchedulerLogger;

import com.sos.JSHelper.Basics.JSVersionInfo;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionFileAge;
import com.sos.JSHelper.io.Files.JSTextFile;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 */
@I18NResourceBundle(baseName = "com_sos_scheduler_messages", defaultLocale = "en")
public class JobSchedulerFileOperationBase extends JobSchedulerJobAdapter {

	private static final String		conOrderParameterSCHEDULER_FILE_PATH							= "scheduler_file_path";
	private static final String		conOrderParameterSCHEDULER_FILE_PARENT							= "scheduler_file_parent";
	private static final String		conOrderParameterSCHEDULER_FILE_NAME							= "scheduler_file_name";
	public static final String		conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_RESULT_SET		= "scheduler_SOSFileOperations_ResultSet";
	public static final String		conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_RESULT_SET_SIZE	= "scheduler_SOSFileOperations_ResultSetSize";
	public static final String		conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_FILE_COUNT		= "scheduler_SOSFileOperations_file_count";

	protected static final String	conPropertyJAVA_IO_TMPDIR										= "java.io.tmpdir";
	
	/** give a path for files to remove */
	protected String				filePath														= System.getProperty(conPropertyJAVA_IO_TMPDIR);

	/** give the number of milliseconds, defaults to 24 hours */
	protected long					lngFileAge														= 86400000;

	/** number of files which causes a warning */
	protected int					warningFileLimit												= 0;

	private static final String		conParameterCREATE_ORDERS_FOR_ALL_FILES							= "create_orders_for_all_files";
	private static final String		conParameterNEXT_STATE											= "next_state";
	private static final String		conParameterORDER_JOBCHAIN_NAME									= "order_jobchain_name";
	private static final String		conParameterCREATE_ORDER										= "create_order";

	protected static final String	conParameterWARNING_FILE_LIMIT									= "warning_file_limit";
	protected static final String	conParameterFILE_AGE											= "file_age";
	protected static final String	conParameterFILE_SPEC											= "file_spec";
	protected static final String	conParameterFILE_SPECIFICATION									= "file_specification";
	protected static final String	conParameterFILE_PATH											= "file_path";

	protected static final String	conParameterMAX_FILE_SIZE										= "max_file_size";
	protected static final String	conParameterMIN_FILE_SIZE										= "min_file_size";
	protected static final String	conParameterMAX_FILE_AGE										= "max_file_age";
	protected static final String	conParameterMIN_FILE_AGE										= "min_file_age";
	protected static final String	conParameterTARGET_FILE											= "target_file";
	protected static final String	conParameterFILE												= "file";
	protected static final String	conParameterSOURCE_FILE											= "source_file";
	protected static final String	conParameterON_EMPTY_RESULT_SET									= "on_empty_result_set";
	protected static final String	conParameterSKIP_LAST_FILES										= "skip_last_files";
	protected static final String	conParameterSKIP_FIRST_FILES									= "skip_first_files";
	protected static final String	conParameterOVERWRITE											= "overwrite";
	protected static final String	conParameterCOUNT_FILES											= "count_files";
	protected static final String	conALL															= "all";
	protected static final String	conTRUE															= "true";
	protected static final String	conParameterREPLACEMENT											= "replacement";
	protected static final String	conParameterREPLACING											= "replacing";
	protected static final String	conParameterRAISE_ERROR_IF_RESULT_SET_IS						= "Raise_Error_If_Result_Set_Is";
	protected static final String	conParameterEXPECTED_SIZE_OF_RESULT_SET							= "Expected_Size_Of_Result_Set";
	protected SOSSchedulerLogger	objSOSLogger													= null;
	@SuppressWarnings("hiding")
	protected Logger				logger															= Logger.getLogger(JobSchedulerFileOperationBase.class);
	protected static final String	conParameterRESULT_LIST_FILE									= "Result_List_File";
	protected static final String	conParameterRECURSIVE											= "recursive";
	protected static final String	conParameterCREATE_DIR											= "create_dir";
	protected static final String	conClassName													= "JobSchedulerFileOperationBase";
	protected static final String	conValueYES														= "yes";
	public static final String		conParameterGRACIOUS											= "gracious";
	@SuppressWarnings("unused")
	private final String			conSVNVersion													= "$Id: JobSchedulerFileOperationBase.java 15640 2011-11-16 11:11:52Z kb $";
	// protected SOSLogger logger = null;
	protected boolean				flgOperationWasSuccessful										= false;
	protected String				name															= null;
	protected String				file															= null;
	private String strFileSpecDefault = ".*";
	protected String				fileSpec														= strFileSpecDefault;
	protected String				minFileAge														= "0";
	protected String				maxFileAge														= "0";
	private String conFileSizeDefault = "-1";
	protected String				minFileSize														= conFileSizeDefault;
	protected String				maxFileSize														= conFileSizeDefault;
	protected int					skipFirstFiles													= 0;
	protected int					skipLastFiles													= 0;
	// Variable_set params = null;
	HashMap<String, String>			params															= null;
	protected String				strGracious														= "false";
	String							source															= null;
	String							target															= null;
	int								flags															= 0;
	String							replacing														= null;
	String							replacement														= null;
	boolean							count_files														= false;
	protected final int				isCaseInsensitive												= Pattern.CASE_INSENSITIVE;
	public int						intNoOfHitsInResultSet											= 0;
	protected String				strOnEmptyResultSet												= null;
	protected String				strResultList2File												= null;
	protected int					intExpectedSizeOfResultSet										= 0;
	// possible values equal, lt, le, eq, ge, gt, ne
	protected String				strRaiseErrorIfResultSetIs										= null;
	protected Vector<File>			lstResultList													= null;																		// new
																																													// Vector<File>();
	protected boolean				flgCreateOrder													= false;
	protected boolean				flgCreateOrders4AllFiles										= false;
	protected String				strOrderJobChainName											= null;
	protected String				strNextState													= null;

	private SOSOptionFileAge objOptionFileAge = null;
	
	public JobSchedulerFileOperationBase() {
		super();
	}
	/**
	* 
	* \brief spooler_init
	* 
	* \details
	*
	* \return 
	*
	* @return
	*/
	public boolean spooler_init() {
		final String conMethodName = conClassName + "::spooler_init";
		boolean flgReturn = super.spooler_init();
		try {
			try {
				this.objSOSLogger = new SOSSchedulerLogger(this.spooler_log);
			}
			catch (Exception e) {
				e.printStackTrace(System.err);
				throw new JobSchedulerException(Messages.getMsg(JSJ_F_0015, "SOSSchedulerLogger ", e.getMessage()));
			}
			return flgReturn;
		}
		catch (Exception e) {
			try {
				if (isNotNull(objSOSLogger))
					objSOSLogger.error(Messages.getMsg(JSJ_F_0016, conMethodName, e.getMessage()));
			}
			catch (Exception x) {
			}
			return false;
		}
		// return flgReturn;
	}

	private void ResetVariables() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::ResetVariables";

		flgOperationWasSuccessful = false;
		name = null;
		file = null;
		fileSpec = strFileSpecDefault;
		minFileAge = "0";
		maxFileAge = "0";
		minFileSize = conFileSizeDefault;
		maxFileSize = conFileSizeDefault;
		skipFirstFiles = 0;
		skipLastFiles = 0;
		strGracious = "false";
		source = null;
		target = null;
		flags = 0;
		replacing = null;
		replacement = EMPTY_STRING;
		count_files = false;
		intNoOfHitsInResultSet = 0;
		strOnEmptyResultSet = null;
		strResultList2File = null;
		intExpectedSizeOfResultSet = 0;
		// possible values equal, lt, le, eq, ge, gt, ne
		strRaiseErrorIfResultSetIs = null;
		lstResultList = null; // new Vector<File>();
		flgCreateOrder = false;
		flgCreateOrders4AllFiles = false;
		strOrderJobChainName = null;
		strNextState = null;

		 objOptionFileAge = new SOSOptionFileAge(null, "file_age", "file_age", "0", "0", false);

	} // private void ResetVariables

	protected boolean getParamBoolean(final String pstrParamName, final boolean pflgDefaultValue) {
		String strReturnValue = getParamValue(pstrParamName);
		boolean flgReturnValue = pflgDefaultValue;
		if (isNotNull(strReturnValue)) {
			try {
				flgReturnValue = SOSFileOperations.toBoolean(strReturnValue);
			}
			catch (Exception e) {
			}
		}
		return flgReturnValue;
	}

	protected long getParamLong(final String pstrParamName, final long plngDefaultValue) {
		long lngReturnValue = getParamInteger(pstrParamName);
		if (lngReturnValue == 0) {
			lngReturnValue = plngDefaultValue;
		}
		return lngReturnValue;
	}

	protected int getParamInteger(final String pstrParamName, final int pintDefaultValue) {
		int intReturnValue = getParamInteger(pstrParamName);
		if (intReturnValue == 0) {
			intReturnValue = pintDefaultValue;
		}
		return intReturnValue;
	}

	protected int getParamInteger(final String pstrParamName) {
		String strT = getParamValue(pstrParamName);
		int intRetVal = 0;
		try {
			if (isNotEmpty(strT)) {
				intRetVal = Integer.parseInt(strT);
			}
		}
		catch (Exception ex) {
			throw new JobSchedulerException(Messages.getMsg(JSJ_E_0130, pstrParamName, ex.getMessage()), ex);
		}
		return intRetVal;
	}

	/**
	 * 
	 * \brief getParamValue - with (alias) (multiple) Parameters
	 * 
	 * \details
	 * the first notEmpty value-string is taken as the value of the parameter and will end the lookup
	 *
	 * \return String
	 *
	 * @param pstrKeys
	 * @return
	 */
	protected String getParamValue(String[] pstrKeys, final String pstrDefaultValue) {
		String strT = pstrDefaultValue;
		for (String strKey : pstrKeys) {
			String strK = getParamValue(strKey);
			if (isNotEmpty(strK)) {
				strT = strK;
				break;
			}
		}
		return strT;
	}

	protected String getParamValue(final String pstrParamName, final String pstrDefaultValue) {
		String strT = getParamValue (pstrParamName);
		if (isNull(strT)) {
			strT = pstrDefaultValue;
		}
		return strT;
	}
	
	protected String getParamValue(final String pstrParamName) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getParamValue";
		/**
		 * space or empty is a valid value of a parameter
		 */
		name = pstrParamName;
		String strT = params.get(pstrParamName);
		if (isNull(strT)) { // not found, try without "underscore"
			strT = params.get(pstrParamName.replaceAll("_", EMPTY_STRING).toLowerCase());
		}
		if (isNotNull(strT)) { // && params.value(pstrParamName).length() > 0) {
			if (strT.length() > 0) {
				logger.info(Messages.getMsg(JSJ_I_0040, pstrParamName, strT));
			}
		}
		return strT;
	} // private String getParamValue

	public boolean initialize(final String pstrVersionInfo) {
		try {
			super.spooler_process();
			logger.info(JSVersionInfo.getVersionString());
			logger.info(pstrVersionInfo);
			params = null;
			params = getSchedulerParameterAsProperties(super.getParameters());
			getParametersFromHashMap();
		}
		catch (Exception e) {
			throw new JobSchedulerException(Messages.getMsg(JSJ_E_0042, e.getLocalizedMessage()), e);
		}
		return true;
	}

	protected void getParametersFromHashMap() {

		ResetVariables();
		
		flgCreateOrders4AllFiles = getParamBoolean(conParameterCREATE_ORDERS_FOR_ALL_FILES, false);
		flgCreateOrder = getParamBoolean(conParameterCREATE_ORDER, false);
		if (flgCreateOrder == true) {
			strOrderJobChainName = getParamValue(conParameterORDER_JOBCHAIN_NAME);
			if (isNull(strOrderJobChainName)) {
				throw new JobSchedulerException(Messages.getMsg(JSJ_E_0020, conParameterORDER_JOBCHAIN_NAME));
			}
			if (spooler.job_chain_exists(strOrderJobChainName) == false) {
				throw new JobSchedulerException(Messages.getMsg(JSJ_E_0041, strOrderJobChainName));
			}
			strNextState = getParamValue(conParameterNEXT_STATE);
		}
		else {
			strOrderJobChainName = null;
			strNextState = null;
			flgCreateOrders4AllFiles = false;
		}

		lngFileAge = calculateFileAge(getParamValue(conParameterFILE_AGE, "0"));
		warningFileLimit = getParamInteger(conParameterWARNING_FILE_LIMIT, 0);

		intExpectedSizeOfResultSet = getParamInteger(conParameterEXPECTED_SIZE_OF_RESULT_SET, 0);
		strRaiseErrorIfResultSetIs = getParamValue(conParameterRAISE_ERROR_IF_RESULT_SET_IS, EMPTY_STRING);
		strResultList2File = getParamValue(conParameterRESULT_LIST_FILE, EMPTY_STRING);
		strOnEmptyResultSet = getParamValue(conParameterON_EMPTY_RESULT_SET, EMPTY_STRING);
		source = file = filePath = getParamValue(new String[] {conParameterSOURCE_FILE, conParameterFILE, conParameterFILE_PATH}, EMPTY_STRING);
		fileSpec = getParamValue(new String[] {conParameterFILE_SPEC, conParameterFILE_SPECIFICATION}, strFileSpecDefault);
		if (isNotEmpty(fileSpec) || isNotEmpty(source)) {
			
		}
		else {  // can be started by a file order source. In this case the name of the file comes as the value of SCHEDULER_WITH_PATH
			source = file = filePath = getParamValue(new String[] {conOrderParameterSCHEDULER_FILE_PATH}, EMPTY_STRING);			
		}
		target = getParamValue(conParameterTARGET_FILE, null);
		
		minFileAge = getParamValue(conParameterMIN_FILE_AGE, "0");
		objOptionFileAge.Value(minFileAge);
		minFileAge = String.valueOf(objOptionFileAge.getAgeAsSeconds());
		
		maxFileAge = getParamValue(conParameterMAX_FILE_AGE, "0");
		objOptionFileAge.Value(maxFileAge);
		maxFileAge = String.valueOf(objOptionFileAge.getAgeAsSeconds());

		minFileSize = getParamValue(conParameterMIN_FILE_SIZE, conFileSizeDefault);
		maxFileSize = getParamValue(conParameterMAX_FILE_SIZE, conFileSizeDefault);
		
		strGracious = getParamValue(conParameterGRACIOUS, "false");
		skipFirstFiles = getParamInteger(conParameterSKIP_FIRST_FILES, 0);
		skipLastFiles = getParamInteger(conParameterSKIP_LAST_FILES, 0);

		flags = 0;
		if (getParamBoolean(conParameterCREATE_DIR, false) == true) {
			flags |= SOSFileOperations.CREATE_DIR;
		}
		if (getParamBoolean(conParameterGRACIOUS, false) == true) {
			flags |= SOSFileOperations.GRACIOUS;
		}
		if (getParamBoolean(conParameterOVERWRITE, false) == false) {
			flags |= SOSFileOperations.NOT_OVERWRITE;
		}
		if (getParamBoolean(conParameterRECURSIVE, false) == true) {
			flags |= SOSFileOperations.RECURSIVE;
		}
		count_files = getParamBoolean(conParameterCOUNT_FILES, false);
		if (count_files == true && isJobchain() == false) {
			logger.warn(Messages.getMsg(JSJ_E_0120, conParameterCOUNT_FILES));
		}
		replacing = getParamValue(conParameterREPLACING, EMPTY_STRING);
		replacement = getParamValue(conParameterREPLACEMENT, EMPTY_STRING);
		String strM = Messages.getMsg(JSJ_E_0110, conParameterREPLACEMENT, conParameterREPLACING);
		if (isNotNull(replacing) && isNull(replacement)) {
			replacement = EMPTY_STRING;
//			throw new JobSchedulerException(strM);
		}
		if (isNull(replacing) && isNotNull(replacement)) {
			throw new JobSchedulerException(strM);
		}
	}

	public void spooler_exit() {
		try {
			super.spooler_exit();
		}
		catch (Exception e) {
			// no error processing at job level
		}
		finally {
		}
	}

	public boolean isGraciousAll() {
		return strGracious != null && strGracious.equalsIgnoreCase(conALL);
	}

	public boolean isGraciousTrue() {
		boolean flgResult = false;
		try {
			if (isNotEmpty(strGracious)) {
				flgResult = SOSFileOperations.toBoolean(strGracious);
			}
		}
		catch (Exception e) {
			logger.error(StackTrace2String(e));
		}
		return flgResult;
	}

	/**
	 * 
	 * \brief saveResultList
	 * 
	 * \details
	 *
	 * \return void
	 *
	 */
	public void saveResultList() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::saveResultList";

		if (isNull(lstResultList)) {
			lstResultList = new Vector<File>();
		}

		Vector<File> lstR = SOSFileOperations.lstResultList;
		if (isNotNull(lstR) && lstR.size() > 0) {
			lstResultList.addAll(lstR);
		}

	} // private void saveResultList

	public boolean createResultListParam(final boolean pflgResult) {
		String strFirstFile = EMPTY_STRING;
		String strT = EMPTY_STRING;
		if (isNull(lstResultList)) {
			saveResultList();
		}
		intNoOfHitsInResultSet = 0;

		if (isNotNull(lstResultList) && lstResultList.size() > 0) {
			intNoOfHitsInResultSet = lstResultList.size();
			strFirstFile = lstResultList.get(0).getAbsolutePath();
			for (File objFile : lstResultList) {
				strT += objFile.getAbsolutePath() + ";";
			}
		}

		if (isJobchain()) {
			if (count_files) {
				setOrderParameter(conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_FILE_COUNT, String.valueOf(intNoOfHitsInResultSet));
			}
			Variable_set objP = spooler_task.order().params();
			if (isNotNull(objP)) {
				setOrderParameter(conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_RESULT_SET, strT);
				setOrderParameter(conOrderParameterSCHEDULER_SOS_FILE_OPERATIONS_RESULT_SET_SIZE, String.valueOf(intNoOfHitsInResultSet));
			}

			if (isNotEmpty(strOnEmptyResultSet) && intNoOfHitsInResultSet <= 0) {
				logger.info(String.format(Messages.getMsg(JSJ_I_0090), strOnEmptyResultSet));
				spooler_task.order().set_state(strOnEmptyResultSet);
			}
		}

		if (isNotEmpty(strResultList2File) && isNotEmpty(strT)) {
			JSTextFile objResultListFile = new JSTextFile(strResultList2File);
			try {
				if (objResultListFile.canWrite()) {
					objResultListFile.Write(strT);
					objResultListFile.close();
				}
				else {
					throw new JobSchedulerException(Messages.getMsg(JSJ_F_0090, conParameterRESULT_LIST_FILE, strResultList2File));
				}
			}
			catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace(System.err);
				logger.fatal(Messages.getMsg(JSJ_F_0080, strResultList2File, conParameterRESULT_LIST_FILE));
			}
		}

		if (isNotEmpty(strRaiseErrorIfResultSetIs)) {
			boolean flgR = compareIntValues(strRaiseErrorIfResultSetIs, intNoOfHitsInResultSet, intExpectedSizeOfResultSet);
			if (flgR == true) {
				logger.info(Messages.getMsg(JSJ_E_0040, intNoOfHitsInResultSet, strRaiseErrorIfResultSetIs, intExpectedSizeOfResultSet));
				return false;
			}
		}

		if (intNoOfHitsInResultSet > 0 && flgCreateOrder == true) {
			if (flgCreateOrders4AllFiles == true) {
				for (File objFile : lstResultList) {
					createOrder(objFile.getAbsolutePath(), strOrderJobChainName);
				}
			}
			else {
				createOrder(strFirstFile, strOrderJobChainName);
			}
		}

		return pflgResult;
	}// private void createResultListParam

	public boolean compareIntValues(final String pstrComparator, int pintValue1, int pintValue2) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::compareIntValues";

		HashMap<String, Integer> objRelOp = new HashMap<String, Integer>();

		objRelOp.put("eq", 1);
		objRelOp.put("equal", 1);
		objRelOp.put("==", 1);
		objRelOp.put("=", 1);
		objRelOp.put("ne", 2);
		objRelOp.put("not equal", 2);
		objRelOp.put("!=", 2);
		objRelOp.put("<>", 2);
		objRelOp.put("lt", 3);
		objRelOp.put("less than", 3);
		objRelOp.put("<", 3);
		objRelOp.put("le", 4);
		objRelOp.put("less or equal", 4);
		objRelOp.put("<=", 4);
		objRelOp.put("ge", 5);
		objRelOp.put(Messages.getMsg(JSJ_T_0010), 5);
		objRelOp.put(">=", 5);
		objRelOp.put("gt", 6);
		objRelOp.put("greater than", 6);
		objRelOp.put(">", 6);

		boolean flgR = false;
		String strT1 = pstrComparator;
		Integer iOp = objRelOp.get(strT1.toLowerCase());
		if (isNotNull(iOp)) {
			switch (iOp) {
				case 1:
					flgR = pintValue1 == pintValue2;
					break;

				case 2:
					flgR = pintValue1 != pintValue2;
					break;

				case 3:
					flgR = pintValue1 < pintValue2;
					break;

				case 4:
					flgR = pintValue1 <= pintValue2;
					break;

				case 5:
					flgR = pintValue1 >= pintValue2;
					break;

				case 6:
					flgR = pintValue1 > pintValue2;
					break;

				default:
					break;
			}
		}
		else {
			throw new JobSchedulerException(Messages.getMsg(JSJ_E_0017, pstrComparator));
		}

		return flgR;
	}// private boolean compareIntValues
	private void createOrder(final String pstrOrder4FileName, final String pstrOrderJobChainName) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::createOrder";

		Order objOrder = spooler.create_order();
		Variable_set objOrderParams = spooler.create_variable_set();

		objOrderParams.set_value(conOrderParameterSCHEDULER_FILE_PATH, pstrOrder4FileName);
		objOrderParams.set_value(conOrderParameterSCHEDULER_FILE_PARENT, new File(pstrOrder4FileName).getParent());
		objOrderParams.set_value(conOrderParameterSCHEDULER_FILE_NAME, new File(pstrOrder4FileName).getName());

		if (isNotEmpty(strNextState)) {
			objOrder.set_state(strNextState);
		}
		objOrder.set_params(objOrderParams);
		objOrder.set_id(pstrOrder4FileName);
		objOrder.set_title(Messages.getMsg(JSJ_I_0017, conMethodName));  // "Order created by %1$s"

		Job_chain objJobchain = spooler.job_chain(pstrOrderJobChainName);
		objJobchain.add_order(objOrder);
		String strT = Messages.getMsg(JSJ_I_0018, pstrOrder4FileName, pstrOrderJobChainName);  // "Order '%1$s' created for JobChain '%2$s'."
		if (isNotEmpty(strNextState)) {
			strT += " " + Messages.getMsg(JSJ_I_0019, strNextState);  // "Next State is '%1$s'."
		}
		logger.info(strT);

	} // private void createOrder

	public String replaceVars4(String pstrReplaceIn) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::replaceVars";

		String strParamNameEnclosedInPercentSigns = "^.*%([^%]+)%.*$"; // "^.*%[^%]+%.*$";

		if (isNotNull(pstrReplaceIn)) {
			// To make orderparams available for substitution in orderparam value
			while (pstrReplaceIn.matches(strParamNameEnclosedInPercentSigns)) {
				String p = pstrReplaceIn.replaceFirst(strParamNameEnclosedInPercentSigns, "$1");
				String strPP = "%" + p + "%";
				String s = params.get(p);
				if (isNotNull(s)) {
					s = s.replace('\\', '/');
					pstrReplaceIn = pstrReplaceIn.replaceAll(strPP, s);
					logger.debug(Messages.getMsg(JSJ_D_0044, name, strPP, s));
				}
				else {
					pstrReplaceIn = pstrReplaceIn.replaceAll(strPP, "?" + p + "?");
					logger.warn(Messages.getMsg(JSJ_W_0043, p));
				}
			}
		}
		return pstrReplaceIn;
	} // private String replaceVars

	/**
	 * 
	 * \brief setParams
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param pobjparams
	 */
	public void setParams(final HashMap<String, String> pobjparams) {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setParams";

		this.params = pobjparams;

	} // private void setParams

	/**
	 * 
	 * \brief setReturnResult
	 * 
	 * \details
	 *
	 * \return boolean
	 *
	 * @param pflgResult
	 * @return
	 */
	public boolean setReturnResult(final boolean pflgResult) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setReturnResult";
		boolean rc1 = pflgResult;
		
//		if (isJobchain()) {
//			rc1 = createResultListParam(pflgResult);
//		}
		// nicht nur fr Jobchains rufen, weil hier auch die Order(s) erzeugt werden
		rc1 = createResultListParam(pflgResult);

		if (rc1 == false && isGraciousAll()) {
			return signalSuccess();
		}
		else {
			if (rc1 == false && isGraciousTrue()) {
				if (isJobchain()) {
					return signalFailure();
				}
				return conJobSuccess;
			}
			else {
				if (rc1 == true) {
					return signalSuccess();
				}
				else {
					return signalFailure();
				}
			}
		}
	} // private boolean setReturnResult

	public void CheckMandatoryFile() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::CheckMandatoryFile";
		if (isNull(file)) {
			throw new JobSchedulerException(String.format(Messages.getMsg(JSJ_E_0020), conParameterFILE));
		}
	} // private void CheckMandatoryFile

	public void CheckMandatorySource() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::CheckMandatoryFile";
		if (isNull(source)) {
			throw new JobSchedulerException(String.format(Messages.getMsg(JSJ_E_0020), conParameterSOURCE_FILE));
		}
	} // private void CheckMandatoryFile

	public void CheckMandatoryTarget() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::CheckMandatoryFile";
		if (isNull(target)) {
			throw new JobSchedulerException(String.format(Messages.getMsg(JSJ_E_0020), conParameterTARGET_FILE));
		}
	} // private void CheckMandatoryFile

	/**
	 * 
	 * \brief calculateFileAge
	 * 
	 * \details
	 *
	 * \return long
	 *
	 * @param hoursMinSec
	 * @return
	 */
	public long calculateFileAge(String hoursMinSec) {
		// TODO implement this method in JSFile
		long age = 0;
		if (isNotEmpty(hoursMinSec)) {
			if (hoursMinSec.indexOf(":") > -1) {
				String[] timeArray = hoursMinSec.split(":");
				long hours = Long.parseLong(timeArray[0]);
				long minutes = Long.parseLong(timeArray[1]);
				long seconds = 0;
				if (timeArray.length > 2) {
					seconds = Long.parseLong(timeArray[2]);
				}
				age = hours * 3600000 + minutes * 60000 + seconds * 1000;
			}
			else {
				age = Long.parseLong(hoursMinSec);
			}
		}

		return age;
	}

	/**
	 * @return the params
	 */
	public Variable_set getParams() {
		final String conMethodName = conClassName + "::getParams";
		throw new JobSchedulerException(Messages.getMsg(JSJ_F_0110, conMethodName));  // not implemented
	}

	/**
	 * @param params1 the params to set
	 */
	public void setParams(Variable_set params1) {
		final String conMethodName = conClassName + "::setParams";
		throw new JobSchedulerException(Messages.getMsg(JSJ_F_0110, conMethodName));  // notImplemented
		// this.params = params1;
	}
private String JSJ_F_0110 = "JSJ_F_0110"; // "This function is not implemented: %1$s";
}
