/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// sqlfile.h

#ifndef __SQLFILE_H
#define __SQLFILE_H

struct Sql_text_format : Text_format
{
                                    Sql_text_format() : Text_format(), _field_quote( '\0' ) {}

    void                            sql_dialect ( const Sos_string& dialect ) { if ( dialect == "odbc" ) field_quote( '\"' ); else field_quote( '\0' ); }

    Text_format&                    field_quote              ( char c )   { _field_quote = c; return *this; }
    char                            field_quote              () const     { return _field_quote; }
 private:
    char                            _field_quote;

};

struct Sql_file : Abs_file
{
                                    Sql_file();
                                    //Sql_file( const char*, Open_mode, const File_spec& );
                                   ~Sql_file();

    void                            open( const char*, Open_mode, const File_spec& );
    void                            close( Close_mode );

//    void                            store( const Const_area& area );
    void                            insert( const Const_area& area );
    void                            update( const Const_area& area );
    void                            store( const Const_area& area );

    void                            set( const Key& );
    void                            del( const Key& key );
    void                            del();

  //Record_position                 key_position (Key::Number = 0) { return _key_position; };
  //Record_length                   key_length   (Key::Number = 0) { return _key_length; };

    static const Abs_file_type&     static_file_type();

    const Sos_string&               tablename();

 protected:
    void                            get_record( Area& area );
    void                            get_record_key( Area& area, const Key& key );

    void                            put_record( const Const_area& area );

 private:
    void                            _init();
    void                            _open_db();
    void                            _replace_select_all( Dynamic_area* );
    void                            _modify_fields( const Sos_string& );

    void                            rollback();
    void                            commit();

    void                            print_value( const Field_descr& f, const Byte* p, ostream* s, Area* hilfspuffer );
    void                            print_field_names( ostream* s, const Field_type& record_type, Bool=false, const char* = "" );
    //void                            print_update_field_names( ostream* s, const Field_type& record_type,
    //                                                          const Const_area& area, const char* name = "", int offs = 0 );
    void                            print_update_field_names( ostream*, const Field_descr&,
                                                              const Const_area& record );
    void                            print_update_field_names( ostream*, const Record_type&,
                                                              const Const_area& record );


    void                            print_where_field_names( ostream* s, const Const_area& area );

    void                            build_update( ostream*, const Const_area& area );
    void                            build_insert( ostream*, const Const_area& area );


    Fill_zero                      _zero_;
    Dynamic_area                   _current_key;

    Sos_string                     _tablename;
    Any_file                       _file;
    Open_mode                      _open_mode;
    Sos_string                     _filename;
    File_spec                      _file_spec;
    Bool                           _opened;
    Bool                           _odbc_mode;

    int                            _required_length;        // minimale Satzlnge bis zum Schlssel

    //jz30.11.95 const Field_type*              _record_type_ptr;
    //jz30.11.95 const Field_descr*             _key_descr_ptr;
    Sos_ptr<Record_type>           _record_type_ptr;
    Sos_ptr<Record_type>           _write_record_type_ptr;
    Sos_ptr<Field_descr>           _key_descr_ptr;
    Sos_ptr<Record_type>           _key_type;           // wenn _key_descr_ptr->field_count() > 0
    Sql_text_format                _format;

    Bool                           _auto_commit;
    int                            _store;
    Bool                           _commit_at_end;
    Bool                           _store_by_delete;
    Bool                           _single_select;
    Bool                           _null_value_optimizing;
    Bool                           _empty_is_null;
};

#endif
