/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
//#define MODULE_NAME "storefl"
//#define AUTHOR      "Joacim Zschimmer"
//#define COPYRIGHT   "1995 SOS GmbH Berlin"

#include "precomp.h"
#include "../kram/sysdep.h"
#include "../kram/sos.h"
#include "../file/absfile.h"

#include "../kram/log.h"

namespace sos {
using namespace std;


//----------------------------------------------------------------------------------Store_file

struct Store_file : Abs_file
{
                                    Store_file         ();
                                   ~Store_file         ();
    void                            open                ( const char*, Open_mode, const File_spec& );
    void                            close               ( Close_mode );

  protected:
    void                            put_record          ( const Const_area& );

  private:
    void                            print_field_names   ( ostream*, const Field_type&, const char* = "" );

    Fill_zero                      _zero_;
    Any_file                       _file;
};

//----------------------------------------------------------------------Store_file_type

struct Store_file_type : Abs_file_type
{
                                Store_file_type         () : Abs_file_type() {}
    virtual const char*         name                    () const { return "store"; }
    virtual Sos_ptr<Abs_file>   create_base_file        () const
    {
        Sos_ptr<Store_file> f = SOS_NEW_PTR( Store_file() );
        return +f;
    }
};

const Store_file_type  _store_file_type;
const Abs_file_type&    store_file_type = _store_file_type;

// --------------------------------------------------------------------Store_file::Store_file

Store_file::Store_file()
:
    _zero_(this+1)
{
}

// -------------------------------------------------------------------Store_file::~Store_file

Store_file::~Store_file()
{
}

// -------------------------------------------------------------------------Store_file::open

void Store_file::open( const char* filename, Open_mode open_mode, const File_spec& file_spec )
{
    _file.obj_owner( this );
    if( !( open_mode & out ) )  throw_xc( "D126" );
    _file.open( filename, Open_mode( open_mode | inout ), file_spec );
}

// --------------------------------------------------------------------------Store_file::close

void Store_file::close( Close_mode close_mode )
{
    _file.close( close_mode );
}

// ---------------------------------------------------------------------Store_file::put

void Store_file::put_record( const Const_area& record )
{
    _file.store( record );
}

} //namespace sos

