/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// subrange.h
#if 0







#ifndef __SUBRANGE_H
#define __SUBRANGE_H

//#pragma implementation

#if !defined __ASSERT_H
#   include <assert.h>
#endif

namespace sos
{

//struct Subrange_error : Xc { Subrange_error( long value, long mini, long maxi ); };
typedef Xc Subrange_error;

void subrange_fail(
    const char* source,
    long        value,
    const char* typ,
    long        mini,
    long        maxi
);


template< class T, int mini, int maxi >
struct Const_subrange
{
    Const_subrange( T t ) : _value( t )  { check( t ); }
    Const_subrange( const Const_subrange& s ) : _value( s._value )  {}

    operator T () const { return _value; }

    static Const_subrange min()  { return mini; }
    static Const_subrange max()  { return maxi; }

    static void check( T t )
    {
#		ifdef __BORLANDC__
#			pragma option -w-ccc
#		endif

        if (t < mini  ||  t > maxi) {
            subrange_fail( SOURCE, t, "", mini, maxi );
        }

#		ifdef __BORLANDC__
#			pragma warn -wccc.
#		endif

        // assert( t>=mini && t<=maxi ); }
     }

  protected:
    Const_subrange()  {}

    T _value;
};


template< class T, int mini, int maxi >
struct Subrange : Const_subrange<T,mini,maxi>
{
    Subrange() {}
    Subrange( T t )  { set( t ); }
    Subrange( const Subrange& s ) : Const_subrange<T,mini,maxi> ( s )  {}
    Subrange( const Const_subrange<T,mini,maxi>& s ) : Const_subrange<T,mini,maxi> ( s )  {}

    Subrange& operator +=  ( T t )   { *this = _value +=  t; return *this; }
    Subrange& operator -=  ( T t )   { *this = _value -=  t; return *this; }
    Subrange& operator *=  ( T t )   { *this = _value *=  t; return *this; }
    Subrange& operator /=  ( T t )   { *this = _value /=  t; return *this; }
    Subrange& operator <<= ( int i ) { *this = _value <<= i; return *this; }
    Subrange& operator >>= ( int i ) { *this = _value >>= i; return *this; }
    Subrange& operator ++  ()        { *this = _value + 1;   return *this; }
    Subrange& operator --  ()        { *this = _value - 1;   return *this; }
    Subrange  operator ++  ( int   ) { Subrange<T,mini,maxi> s = *this; ++*this; return s; }
    Subrange  operator --  ( int   ) { Subrange<T,mini,maxi> s = *this; --*this; return s; }

    // Fr Schleifen: (#define loop_in_subrange)
    Subrange& step         ()        { _value++; return *this; }
    Bool      overflowed   ()        { return _value > maxi; }
    operator T () const { return _value; }

  private:
    void set( T t )  { check( t ); _value = t; }
};

#define loop_in_subrange(i)  for (i = i.min(); !i.overflowed(); i.step() )


} //namespace sos

#endif


#endif
