/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: thread_semaphore.cxx 11394 2005-04-03 08:30:29Z jz $

#include "precomp.h"
#include "sos.h"
#include "thread_semaphore.h"
#include "log.h"
#include "msec.h"

#if 0

namespace sos {

//-------------------------------------------------------------------------------------------static
#ifdef SYSTEM_WIN

typedef BOOL (WINAPI *Proc)(LPCRITICAL_SECTION);

static volatile Proc      tryentercriticalsection = (Proc)-1;
static volatile HINSTANCE kernel32;

#endif    
//-------------------------------------------------------------------Thread_semaphore::Guard::Guard
/*
Thread_semaphore::Guard_with_log::Guard_with_log( Thread_semaphore* s, const string& debug )    
: 
    Guard(s),
    _debug(debug) 
{
    LOG( "Semaphore " << debug << " entered\n" ); 
    log_indent(+1);
}

//------------------------------------------------------------------Thread_semaphore::Guard::~Guard

Thread_semaphore::Guard_with_log::~Guard_with_log()    
{ 
    log_indent(-1);
    LOG( "Semaphore " << _debug << " leaved\n" ); 
}
*/
//-------------------------------------------------------------------------------------------------

void Thread_semaphore_with_log::enter()
{
    int ok = false;

    if( log_ptr )
    {
        ok = Thread_semaphore::try_enter();
        if( !ok )  LOG( "Semaphore " << _name << " locked. Waiting ...\n" );
    }
    
    string suffix;

    if( !ok )  
    {
        int start = elapsed_msec();
        
        Thread_semaphore::enter();
     
        int wait_msec = elapsed_msec() - start;
        if( wait_msec > 1 )  suffix = " after " + as_string(wait_msec/1000.0) + "s";
    }

    LOG( "Semaphore " << _name << " entered" << suffix << '\n' ); 
    log_indent(+1);
}

//-------------------------------------------------------------------------------------------------

void Thread_semaphore_with_log::leave()
{
    log_indent(-1);
    LOG( "Semaphore " << _name << " leave\n" ); 
    Thread_semaphore::leave();
}

//----------------------------------------------------------------------Thread_semaphore::try_enter
#ifdef SYSTEM_WIN

bool Thread_semaphore::try_enter()
{
    bool ok = false;

    if( tryentercriticalsection == (Proc)-1 )
    {
        OSVERSIONINFO v;  memset( &v, 0, sizeof v ); v.dwOSVersionInfoSize = sizeof v;

        GetVersionEx( &v );
        if( v.dwPlatformId == VER_PLATFORM_WIN32_NT )  // Windows NT oder Windows 2000?
        {
            kernel32 = LoadLibrary( "kernel32.dll" );
            if( kernel32 )  tryentercriticalsection = (Proc)GetProcAddress( kernel32, "TryEnterCriticalSection" );
        }

        if( tryentercriticalsection == (Proc)-1 )  tryentercriticalsection = NULL;
    }

    Proc t = tryentercriticalsection;
    if( t != NULL  &&  t != (Proc)-1 )  
    {
        ok = t( &_semaphore ) != 0;
    }

    return ok;
}

#endif
//-------------------------------------------------------------------------------------------------

} //namespace sos
#endif
