/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: java_com.h 14174 2011-01-13 16:26:26Z ss $

#ifndef __ZSCHIMMER_JAVA_COM_H
#define __ZSCHIMMER_JAVA_COM_H


#include "java.h"
#include "z_com_server.h"

namespace zschimmer {
namespace javabridge {

struct Java_idispatch_container;

//------------------------------------------------------------------------------------------Com_env

struct Com_env : Env
{
                                Com_env                     ( JNIEnv* jenv = javabridge::jni_env() ) : Env( jenv ) {}

    void                        jstring_to_bstr             ( const jstring&, BSTR* );
    jobject                     jobject_from_variant        ( const VARIANT&, Java_idispatch_container* = NULL );
    //jobject                   jobject_from_idispatch      ( IDispatch* );
    DISPID                      java_com_get_dispid         ( jlong jidispatch, jstring jname, WORD* dispatch_context = NULL );
    com::Variant                variant_java_com_call       ( jclass, jlong jidispatch, jstring name                 , jobjectArray jparams );
    com::Variant                variant_java_com_call       ( jclass, jlong jidispatch, DISPID, WORD dispatch_context, jobjectArray jparams );
    jobject                     jobject_java_com_call       ( jclass, jlong jidispatch, jstring name                 , jobjectArray jparams );
    jobject                     jobject_java_com_call       ( jclass, jlong jidispatch, DISPID, WORD dispatch_context, jobjectArray jparams );
    com::Variant                variant_java_com_call       ( jclass, jlong jidispatch, jstring name                 , jstring      par1 );
    com::Variant                variant_java_com_call       ( jclass, jlong jidispatch, DISPID, WORD dispatch_context, jstring      par1 );
    com::Variant                jobject_to_variant          (jobject jparam);
    bool                        jobjectarray_to_dispparams  ( jobjectArray jparams, com::Dispparams* );
    com::Variant                variant_java_com_call       ( jclass, jlong jidispatch, DISPID dispid, WORD context, com::Dispparams* );

    jstring                     jstring_from_bstr           ( const BSTR bstr )                     { return jni_env()->NewString( (const jchar*)bstr, SysStringLen(bstr) );}
    jstring                     jstring_from_variant        ( const VARIANT& );
};

//-----------------------------------------------------------------------------------Java_idispatch

struct Java_idispatch : Object
{
    Z_GNU_ONLY(                 Java_idispatch              ();  )                                  // Fr gcc 3.2. Nicht implementiert.
                                Java_idispatch              ( IDispatch*, bool is_owner, const char* subclass );
                               ~Java_idispatch              ();

    jobject                 get_jobject                     ()                                      { return _jobject; }


    ptr<IDispatch>             _idispatch;
  //string                     _class_name;
    jobject                    _jobject;
    Global_jobject             _global_jobject;
    bool                       _is_owner;                   // Java-Idispatch wird mit Aufruf von ~Java_idispatch() ungltig
};

//-------------------------------------------------------------------------Java_idispatch_container

struct Java_idispatch_container
{
                                Java_idispatch_container    ()                                      : _zero_(this+1) {}

    void                        add_object                  ( Java_idispatch* );
    void                        release_objects             ()                                      { _java_idispatch_list.clear(); }


    Fill_zero                  _zero_;

    std::list< ptr<Java_idispatch> > _java_idispatch_list;        // Hlt alle in einer nativen Methode erzeugten IDispatchs, bis release_objects()
};

//--------------------------------------------------------------------------------Jobject_idispatch

Z_DEFINE_IID( Jobject_idispatch, "feee46eb-6c1b-11d8-8103-000476ee8afb", 
                                  feee46eb,6c1b,11d8,81,03,00,04,76,ee,8a,fb );

struct Jobject_idispatch : Object, //com::simple_idispatch_implementation< Jobject_idispatch >,
                           Non_cloneable
{
                                Jobject_idispatch           ( jobject );
                               ~Jobject_idispatch           ();

    STDMETHODIMP                QueryInterface              ( const IID& iid, void** result )
    {
        Z_IMPLEMENT_QUERY_INTERFACE( this, iid, Jobject_idispatch, result );

        return Object::QueryInterface( iid, result );
    }


    jobject                    _jobject;
};

//-------------------------------------------------------------------------------------------------

} //namespace javabridge
} //namespace zschimmer


#endif

